@@ -201,7 +201,8 @@ class Bamboo(AtlassianRestAPI):
         :return:
         """
         resource = 'plan/{plan_key}/vcsBranches'.format(plan_key=plan_key)
-        return self.base_list_call(resource, start_index=0, max_results=max_results)
+        return self.base_list_call(resource, start_index=0, max_results=max_results, clover_enabled=None, expand=None,
+                                   favourite=None)
 
     """ Build results """
 
@@ -406,6 +407,25 @@ class Bamboo(AtlassianRestAPI):
         resource = "result/{}-{}-{}/label/{}".format(project_key, plan_key, build_number, label)
         return self.delete(self.resource_url(resource))
 
+    def get_projects(self):
+        """Method used to list all projects defined in Bamboo.
+        Projects without any plan are not listed by default, unless showEmpty query param is set to true."""
+        resource = 'project?showEmpty'
+        for project in self.get(self.resource_url(resource)):
+            yield project
+
+    def get_project(self, project_key):
+        """Method used to retrieve information for project specified as project key.
+        Possible expand parameters: plans, list of plans for project. plans.plan, list of plans with plan details
+        (only plans visible - READ permission for user)"""
+        resource = 'project/{}?showEmpty'.format(project_key)
+        return self.get(self.resource_url(resource))
+
+    def delete_project(self, project_key):
+        """Marks project for deletion. Project will be deleted by a batch job."""
+        resource = 'project/{}'.format(project_key)
+        return self.delete(self.resource_url(resource))
+
     """ Deployments """
 
     def deployment_projects(self):
@@ -590,6 +610,20 @@ class Bamboo(AtlassianRestAPI):
             params['height'] = height
         return self.get(self.resource_url('chart'), params=params)
 
+    def reindex(self):
+        """
+        Returns status of the current indexing operation.
+        reindexInProgress - reindex is currently performed in background reindexPending - reindex is required
+        (i.e. it failed before or some upgrade task asked for it)
+        """
+        return self.get(self.resource_url('reindex'))
+
+    def stop_reindex(self):
+        """
+        Kicks off a reindex. Requires system admin permissions to perform this reindex.
+        """
+        return self.post(self.resource_url('reindex'))
+
     def health_check(self):
         """
         Get health status
